@@ -15,8 +15,10 @@ from sentry.models import TeamMember, MEMBER_OWNER, \
   ProjectKey, Team
 from sentry.permissions import can_create_projects, can_remove_project
 from sentry.plugins import plugins
+from sentry.plugins.helpers import set_option, get_option
 from sentry.web.decorators import login_required, has_access
-from sentry.web.forms import EditProjectForm, RemoveProjectForm
+from sentry.web.forms import EditProjectForm, RemoveProjectForm, \
+  EditProjectAdminForm
 from sentry.web.helpers import render_to_response, get_project_list, \
   plugin_config, get_team_list
 
@@ -97,25 +99,30 @@ def manage_project(request, project):
 
     team_list = get_team_list(request.user)
 
-    form = EditProjectForm(request, team_list, request.POST or None, instance=project)
+    if request.user.has_perm('sentry.can_change_project'):
+        form_cls = EditProjectAdminForm
+    else:
+        form_cls = EditProjectForm
+
+    form = form_cls(request, team_list, request.POST or None, instance=project, initial={
+        'origins': '\n'.join(get_option('sentry:origins', project) or []),
+        'owner': project.owner_id,
+    })
 
     if form.is_valid():
         project = form.save()
-
+        set_option('sentry:origins', form.cleaned_data.get('origins') or [], project)
         return HttpResponseRedirect(request.path + '?success=1')
 
-    member_list = [(tm, tm.user) for tm in project.team.member_set.select_related('user')]
-
-    try:
-        key = ProjectKey.objects.get(user=request.user, project=project)
-    except ProjectKey.DoesNotExist:
-        key = None  # superuser
+    if not project.team:
+        member_list = []
+    else:
+        member_list = [(tm, tm.user) for tm in project.team.member_set.select_related('user')]
 
     context = csrf(request)
     context.update({
         'can_remove_project': can_remove_project(request.user, project),
         'page': 'details',
-        'key': key,
         'form': form,
         'project': project,
         'member_list': member_list,
@@ -125,6 +132,28 @@ def manage_project(request, project):
     return render_to_response('sentry/projects/manage.html', context, request)
 
 
+@login_required
+@has_access
+def client_help(request, project):
+    try:
+        key = ProjectKey.objects.get(user=request.user, project=project)
+    except ProjectKey.DoesNotExist:
+        key = None  # superuser
+
+    context = {
+        'can_remove_project': can_remove_project(request.user, project),
+        'page': 'client_help',
+        'project': project,
+        'key': key,
+    }
+    if key:
+        context.update({
+            'dsn': key.get_dsn(),
+            'dsn_public': key.get_dsn(public=True),
+        })
+    return render_to_response('sentry/projects/client_help.html', context, request)
+
+
 @login_required
 @has_access(MEMBER_OWNER)
 @csrf_protect
